@@ -50,7 +50,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "auth", .type = OPTION_MULTI_LINE, .mandatory = 1 },
 	{ .name = "route", .type = OPTION_MULTI_LINE, .mandatory = 0 },
 	{ .name = "listen-host", .type = OPTION_STRING, .mandatory = 0 },
-	{ .name = "tcp-port", .type = OPTION_NUMERIC, .mandatory = 1 },
+	{ .name = "tcp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "udp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "keepalive", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "dpd", .type = OPTION_NUMERIC, .mandatory = 0 },
@@ -72,8 +72,11 @@ static struct cfg_options available_options[] = {
 	{ .name = "pid-file", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "socket-file", .type = OPTION_STRING, .mandatory = 1 },
 	{ .name = "banner", .type = OPTION_STRING, .mandatory = 0 },
+	/* this is alias for cisco-client-compat */
 	{ .name = "always-require-cert", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "cisco-client-compat", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "use-utmp", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "use-dbus", .type = OPTION_BOOLEAN, .mandatory = 1 },
 	{ .name = "try-mtu-discovery", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "ping-leases", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "tls-priorities", .type = OPTION_STRING, .mandatory = 0 },
@@ -165,6 +168,7 @@ unsigned j;
 			else \
 				s_name = 0; \
 		} \
+		free(tmp_tf); \
 	}
 
 #define READ_NUMERIC(name, s_name) \
@@ -229,6 +233,7 @@ unsigned j, mand;
 char** auth = NULL;
 unsigned auth_size = 0;
 unsigned prefix = 0;
+unsigned force_cert_auth;
 
 	pov = configFileLoad(file);
 	if (pov == NULL) {
@@ -326,8 +331,15 @@ unsigned prefix = 0;
 	READ_STRING("socket-file", config->socket_file_prefix);
 
 	READ_STRING("banner", config->banner);
-	READ_TF("always-require-cert", config->force_cert_auth, 1);
+	READ_TF("cisco-client-compat", config->cisco_client_compat, 0);
+	READ_TF("always-require-cert", force_cert_auth, 1);
+	if (force_cert_auth == 0) {
+		fprintf(stderr, "note that 'always-require-cert' was replaced by 'cisco-client-compat'\n");
+		config->cisco_client_compat = 1;
+	}
+
 	READ_TF("use-utmp", config->use_utmp, 1);
+	READ_TF("use-dbus", config->use_dbus, 0);
 	READ_TF("try-mtu-discovery", config->try_mtu, 0);
 	READ_TF("ping-leases", config->ping_leases, 0);
 
@@ -430,7 +442,7 @@ static void check_cfg( struct cfg_st *config)
 	}
 
 	if (config->auth_types & AUTH_TYPE_CERTIFICATE) {
-		if (config->force_cert_auth)
+		if (config->cisco_client_compat == 0)
 			config->cert_req = GNUTLS_CERT_REQUIRE;
 		else
 			config->cert_req = GNUTLS_CERT_REQUEST;
@@ -516,7 +528,7 @@ int cmd_parser (int argc, char **argv, struct cfg_st* config)
 }
 
 #define DEL(x) free(x);x=NULL
-void reload_cfg_file(struct cfg_st* config)
+void clear_cfg_file(struct cfg_st* config)
 {
 unsigned i;
 
@@ -565,6 +577,13 @@ unsigned i;
 	DEL(config->cert);
 	DEL(config->network.routes);
 
+	return;
+}
+
+void reload_cfg_file(struct cfg_st* config)
+{
+	clear_cfg_file(config);
+
 	memset(config, 0, sizeof(*config));
 
 	parse_cfg_file(cfg_file, config);
