@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Nikos Mavrogiannopoulos
+ * Copyright (C) 2013, 2014 Nikos Mavrogiannopoulos
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,12 +37,14 @@
 #include <gnutls/x509.h>
 #include <gnutls/crypto.h>
 #include <tlslib.h>
-#include "ipc.h"
 #include "setproctitle.h"
 #ifdef HAVE_LIBWRAP
 # include <tcpd.h>
 #endif
 
+#ifdef HAVE_LIBSYSTEMD_DAEMON
+# include <systemd/sd-daemon.h>
+#endif
 #include <main.h>
 #include <route-add.h>
 #include <main-auth.h>
@@ -74,6 +76,56 @@ static void ms_sleep(unsigned ms)
   nanosleep(&tv, NULL);
 }
 
+static void add_listener(struct listen_list_st *list,
+	int fd, int family, int socktype, int protocol,
+	struct sockaddr* addr, socklen_t addr_len)
+{
+	struct listener_st *tmp;
+
+	tmp = calloc(1, sizeof(struct listener_st));
+	tmp->fd = fd;
+	tmp->family = family;
+	tmp->socktype = socktype;
+	tmp->protocol = protocol;
+	
+	tmp->addr_len = addr_len;
+	memcpy(&tmp->addr, addr, addr_len);
+
+	list_add(&list->head, &(tmp->list));
+	list->total++;
+}
+
+static void set_udp_socket_options(int fd)
+{
+int y;
+#if defined(IP_DONTFRAG)
+	y = 1;
+	if (setsockopt(fd, SOL_IP, IP_DONTFRAG,
+		       (const void *) &y, sizeof(y)) < 0)
+		perror("setsockopt(IP_DF) failed");
+#elif defined(IP_MTU_DISCOVER)
+	y = IP_PMTUDISC_DO;
+	if (setsockopt(fd, IPPROTO_IP, IP_MTU_DISCOVER,
+		       (const void *) &y, sizeof(y)) < 0)
+		perror("setsockopt(IP_DF) failed");
+#endif
+}
+
+static void set_common_socket_options(int fd)
+{
+int val;
+
+	val = fcntl(fd, F_GETFL, 0);
+	if ((val == -1)
+	    || (fcntl(fd, F_SETFL, val | O_NONBLOCK) < 0)) {
+		int e = errno;
+		fprintf(stderr, "fcntl() error: %s", strerror(e));
+		exit(1);
+	}
+
+	set_cloexec_flag (fd, 1);
+}
+
 static 
 int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_list_st *list)
 {
@@ -81,7 +133,6 @@ int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_lis
 	int s, y;
 	const char* type = NULL;
 	char buf[512];
-	struct listener_st *tmp;
 
 	for (ptr = res; ptr != NULL; ptr = ptr->ai_next) {
 		if (ptr->ai_family != AF_INET && ptr->ai_family != AF_INET6)
@@ -93,7 +144,7 @@ int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_lis
 			type = "UDP";
 		else
 			continue;
-			
+
 		if (config->foreground != 0)
 			fprintf(stderr, "listening (%s) on %s...\n",
 				type, human_addr(ptr->ai_addr, ptr->ai_addrlen,
@@ -105,7 +156,7 @@ int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_lis
 			perror("socket() failed");
 			continue;
 		}
-		
+
 #if defined(IPV6_V6ONLY)
 		if (ptr->ai_family == AF_INET6) {
 			y = 1;
@@ -123,20 +174,9 @@ int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_lis
 		}
 
 		if (ptr->ai_socktype == SOCK_DGRAM) {
-#if defined(IP_DONTFRAG)
-			y = 1;
-			if (setsockopt(s, SOL_IP, IP_DONTFRAG,
-				       (const void *) &y, sizeof(y)) < 0)
-				perror("setsockopt(IP_DF) failed");
-#elif defined(IP_MTU_DISCOVER)
-			y = IP_PMTUDISC_DO;
-			if (setsockopt(s, IPPROTO_IP, IP_MTU_DISCOVER,
-				       (const void *) &y, sizeof(y)) < 0)
-				perror("setsockopt(IP_DF) failed");
-#endif
+			set_udp_socket_options(s);
 		}
 
-		set_cloexec_flag (s, 1);
 
 		if (bind(s, ptr->ai_addr, ptr->ai_addrlen) < 0) {
 			perror("bind() failed");
@@ -151,16 +191,11 @@ int _listen_ports(struct cfg_st* config, struct addrinfo *res, struct listen_lis
 			}
 		}
 
-		tmp = calloc(1, sizeof(struct listener_st));
-		tmp->fd = s;
-		tmp->family = ptr->ai_family;
-		tmp->socktype = ptr->ai_socktype;
-		tmp->protocol = ptr->ai_protocol;
-		tmp->addr_len = ptr->ai_addrlen;
-		memcpy(&tmp->addr, ptr->ai_addr, tmp->addr_len);
+		set_common_socket_options(s);
+		
+		add_listener(list, s, ptr->ai_family, ptr->ai_socktype,
+			ptr->ai_protocol, ptr->ai_addr, ptr->ai_addrlen);
 
-		list_add(&list->head, &(tmp->list));
-		list->total++;
 	}
 	
 	fflush(stderr);
@@ -175,11 +210,86 @@ listen_ports(struct cfg_st* config, struct listen_list_st *list, const char *nod
 {
 	struct addrinfo hints, *res;
 	char portname[6];
-	int ret;
+	int ret, fds;
 
 	list_head_init(&list->head);
 	list->total = 0;
 
+#ifdef HAVE_LIBSYSTEMD_DAEMON
+	/* Support for systemd socket-activatable service */
+	if ((fds=sd_listen_fds(0)) > 0) {
+		/* if we get our fds from systemd */
+		unsigned i;
+		int family, type, fd;
+		struct sockaddr_storage tmp_sock;
+		socklen_t tmp_sock_len;
+
+		for (i=0;i<fds;i++) {
+			fd = SD_LISTEN_FDS_START+i;
+
+			if (sd_is_socket(fd, AF_INET, 0, -1))
+				family = AF_INET;
+			else if (sd_is_socket(fd, AF_INET6, 0, -1))
+				family = AF_INET6;
+			else {
+				fprintf(stderr, "Non-internet socket fd received!\n");
+				continue;
+			}
+
+			if (sd_is_socket(fd, 0, SOCK_STREAM, -1))
+				type = SOCK_STREAM;
+			else if (sd_is_socket(fd, 0, SOCK_DGRAM, -1))
+				type = SOCK_DGRAM;
+			else {
+				fprintf(stderr, "Non-TCP or UDP socket fd received!\n");
+				continue;
+			}
+
+			if (type == SOCK_DGRAM)
+				set_udp_socket_options(fd);
+
+			/* obtain socket params */
+			tmp_sock_len = sizeof(tmp_sock);
+			ret = getsockname(fd, (struct sockaddr*)&tmp_sock, &tmp_sock_len);
+			if (ret == -1) {
+				perror("getsockname failed");
+				continue;
+			}
+
+			set_common_socket_options(fd);
+
+			if (type == SOCK_STREAM) {
+				if (family == AF_INET)
+					config->port = ntohs(((struct sockaddr_in*)&tmp_sock)->sin_port);
+				else
+					config->port = ntohs(((struct sockaddr_in6*)&tmp_sock)->sin6_port);
+			} else if (type == SOCK_DGRAM) {
+				if (family == AF_INET)
+					config->udp_port = ntohs(((struct sockaddr_in*)&tmp_sock)->sin_port);
+				else
+					config->udp_port = ntohs(((struct sockaddr_in6*)&tmp_sock)->sin6_port);
+			}
+
+			add_listener(list, fd, family, type, 0, (struct sockaddr*)&tmp_sock, tmp_sock_len);
+		}
+
+		if (list->total == 0) {
+			fprintf(stderr, "no useful sockets were provided by systemd\n");
+			exit(1);
+		}
+
+		if (config->foreground != 0)
+			fprintf(stderr, "listening on %d systemd sockets...\n", list->total);
+		
+		return 0;
+	}
+#endif
+
+	if (config->port == 0) {
+		fprintf(stderr, "tcp-port option is mandatory!\n");
+		return -1;
+	}
+
 	snprintf(portname, sizeof(portname), "%d", config->port);
 
 	memset(&hints, 0, sizeof(hints));
@@ -312,10 +422,10 @@ struct script_wait_st *stmp = NULL, *spos;
 		list_for_each_safe(&s->script_list.head, stmp, spos, list) {
 			if (stmp->pid == pid) {
 				mslog(s, stmp->proc, LOG_DEBUG, "%s-script exit status: %u", stmp->up?"connect":"disconnect", estatus);
+				list_del(&stmp->list);
 				ret = handle_script_exit(s, stmp->proc, estatus);
 				if (ret < 0)
 					remove_proc(s, stmp->proc, 1);
-				list_del(&stmp->list);
 				free(stmp);
 				break;
 			}
@@ -426,7 +536,7 @@ static void drop_privileges(main_server_st* s)
 #endif
 }
 
-/* clears the server llist and clist. To be used after fork().
+/* clears the server listen_list and proc_list. To be used after fork().
  * It frees unused memory and descriptors.
  */
 void clear_lists(main_server_st *s)
@@ -436,20 +546,20 @@ void clear_lists(main_server_st *s)
 	struct script_wait_st *script_tmp = NULL, *script_pos;
 	struct banned_st *btmp = NULL, *bpos;
 
-	list_for_each_safe(&s->llist.head, ltmp, lpos, list) {
+	list_for_each_safe(&s->listen_list.head, ltmp, lpos, list) {
 		close(ltmp->fd);
 		list_del(&ltmp->list);
 		free(ltmp);
-		s->llist.total--;
+		s->listen_list.total--;
 	}
 
-	list_for_each_safe(&s->clist.head, ctmp, cpos, list) {
+	list_for_each_safe(&s->proc_list.head, ctmp, cpos, list) {
 		if (ctmp->fd >= 0)
 			close(ctmp->fd);
 		list_del(&ctmp->list);
 		memset(ctmp, 0, sizeof(*ctmp));
 		free(ctmp);
-		s->clist.total--;
+		s->proc_list.total--;
 	}
 
 	list_for_each_safe(&s->ban_list.head, btmp, bpos, list) {
@@ -463,9 +573,8 @@ void clear_lists(main_server_st *s)
 	}
 
 	tls_cache_deinit(s->tls_db);
-	
 	ip_lease_deinit(&s->ip_leases);
-
+	ctl_handler_deinit(s);
 }
 
 static void kill_children(main_server_st* s)
@@ -474,7 +583,7 @@ static void kill_children(main_server_st* s)
 
 	/* kill the security module server */
 	kill(s->sec_mod_pid, SIGTERM);
-	list_for_each(&s->clist.head, ctmp, list) {
+	list_for_each(&s->proc_list.head, ctmp, list) {
 		if (ctmp->pid != -1) {
 			kill(ctmp->pid, SIGTERM);
 			user_disconnected(s, ctmp);
@@ -482,13 +591,13 @@ static void kill_children(main_server_st* s)
 	}
 }
 
-static void handle_term(int signo)
+void request_stop(int signo)
 {
 	/* kill all children */
 	terminate = 1;
 }
 
-static void handle_reload(int signo)
+void request_reload(int signo)
 {
 	reload_conf = 1;
 }
@@ -547,10 +656,11 @@ time_t now;
 	/* search for the IP and the session ID in all procs */
 	now = time(0);
 
-	list_for_each(&s->clist.head, ctmp, list) {
-		if (session_id_size == ctmp->session_id_size &&
-			memcmp(session_id, ctmp->session_id, session_id_size) == 0 &&
+	list_for_each(&s->proc_list.head, ctmp, list) {
+		if (session_id_size == ctmp->dtls_session_id_size &&
+			memcmp(session_id, ctmp->dtls_session_id, session_id_size) == 0 &&
 			(now - ctmp->udp_fd_receive_time > UDP_FD_RESEND_TIME)) {
+			UdpFdMsg msg = UDP_FD_MSG__INIT;
 
 			ret = connect(listener->fd, (void*)&cli_addr, cli_addr_size);
 			if (ret == -1) {
@@ -559,7 +669,11 @@ time_t now;
 				return -1;
 			}
 
-			ret = send_udp_fd(s, ctmp, listener->fd);
+			ret = send_socket_msg_to_worker(s, ctmp, CMD_UDP_FD,
+				listener->fd,
+				&msg, 
+				(pack_size_func)udp_fd_msg__get_packed_size,
+				(pack_func)udp_fd_msg__pack);
 			if (ret < 0) {
 				mslog(s, ctmp, LOG_ERR, "error passing UDP socket");
 				return -1;
@@ -585,12 +699,14 @@ fail:
 
 }
 
-#define MAINTAINANCE_TIME(s) ((s)->config->cookie_validity + 300)
+#define MAINTAINANCE_TIME(s) (MIN((300 + MAX_ZOMBIE_SECS), ((s)->config->cookie_validity + 300)))
 
 static void check_other_work(main_server_st *s)
 {
+unsigned total = 10;
+
 	if (reload_conf != 0) {
-		mslog(s, NULL, LOG_INFO, "HUP signal was received; reloading configuration");
+		mslog(s, NULL, LOG_INFO, "reloading configuration");
 		reload_cfg_file(s->config);
 		reload_conf = 0;
 	}
@@ -600,20 +716,35 @@ static void check_other_work(main_server_st *s)
 	}
 
 	if (terminate != 0) {
-		mslog(s, NULL, LOG_DEBUG, "termination signal received; waiting for children to die");
+		mslog(s, NULL, LOG_DEBUG, "termination request received; waiting for children to die");
 		kill_children(s);
-		closelog();
+		while (waitpid(-1, NULL, WNOHANG) == 0) {
+			if (total == 0) {
+				mslog(s, NULL, LOG_DEBUG, "not everyone died; forcing kill");
+				kill(0, SIGKILL);
+			}
+			ms_sleep(500);
+			total--;
+		}
 		remove(s->socket_file);
 		remove_pid_file();
-		while (waitpid(-1, NULL, 0) > 0);
+
+		/* try to clean-up everything allocated to ease checks 
+		 * for memory leaks.
+		 */
+		clear_lists(s);
+		tls_global_deinit(s);
+		clear_cfg_file(s->config);
+		closelog();
 		exit(0);
 	}
 
 	/* Check if we need to expire any cookies */
 	if (need_maintenance != 0) {
 		need_maintenance = 0;
-		mslog(s, NULL, LOG_INFO, "Performing maintenance");
+		mslog(s, NULL, LOG_DEBUG, "Performing maintenance");
 		expire_tls_sessions(s);
+		expire_zombies(s);
 		expire_banned(s);
 		alarm(MAINTAINANCE_TIME(s));
 	}
@@ -642,27 +773,42 @@ int main(int argc, char** argv)
 	int fd, pid, e;
 	struct listener_st *ltmp = NULL;
 	struct proc_st *ctmp = NULL, *cpos;
-	fd_set rd;
-	int val, n = 0, ret, flags;
-	struct timeval tv;
+	struct ctl_handler_st* ctl_tmp = NULL, *ctl_pos;
+	fd_set rd_set, wr_set;
+	int n = 0, ret, flags;
+#ifdef HAVE_PSELECT
+	struct timespec ts;
+#else
+	struct timeval ts;
+#endif
 	int cmd_fd[2];
 	struct worker_st ws;
 	struct cfg_st config;
 	unsigned set;
 	main_server_st s;
+	sigset_t emptyset, blockset;
 
 	memset(&s, 0, sizeof(s));
 
-	list_head_init(&s.clist.head);
+	list_head_init(&s.proc_list.head);
 	list_head_init(&s.ban_list.head);
 	list_head_init(&s.script_list.head);
+	list_head_init(&s.ctl_list.head);
 	tls_cache_init(&s.tls_db);
 	ip_lease_init(&s.ip_leases);
-	
-	ocsignal(SIGINT, handle_term);
-	ocsignal(SIGTERM, handle_term);
+
+	sigemptyset(&blockset);
+	sigemptyset(&emptyset);
+	sigaddset(&blockset, SIGALRM);
+	sigaddset(&blockset, SIGTERM);
+	sigaddset(&blockset, SIGINT);
+	sigaddset(&blockset, SIGCHLD);
+	sigaddset(&blockset, SIGHUP);
+
+	ocsignal(SIGINT, request_stop);
+	ocsignal(SIGTERM, request_stop);
 	ocsignal(SIGPIPE, SIG_IGN);
-	ocsignal(SIGHUP, handle_reload);
+	ocsignal(SIGHUP, request_reload);
 	ocsignal(SIGCHLD, handle_children);
 	ocsignal(SIGALRM, handle_alarm);
 
@@ -694,7 +840,7 @@ int main(int argc, char** argv)
 	main_auth_init(&s);
 
 	/* Listen to network ports */
-	ret = listen_ports(&config, &s.llist, config.name);
+	ret = listen_ports(&config, &s.listen_list, config.name);
 	if (ret < 0) {
 		fprintf(stderr, "Cannot listen to specified ports\n");
 		exit(1);
@@ -711,7 +857,6 @@ int main(int argc, char** argv)
 	allow_severity = LOG_DAEMON|LOG_INFO;
 	deny_severity = LOG_DAEMON|LOG_WARNING;
 #endif	
-	memset(&ws, 0, sizeof(ws));
 
 	if (config.foreground == 0) {
 		if (daemon(0, 0) == -1) {
@@ -729,53 +874,75 @@ int main(int argc, char** argv)
 	tls_global_init_certs(&s);
 
 	mslog(&s, NULL, LOG_INFO, "initialized %s", PACKAGE_STRING);
+
+	ret = ctl_handler_init(&s);
+	if (ret < 0) {
+		fprintf(stderr, "Cannot create command handler\n");
+		exit(1);
+	}
+
+	sigprocmask(SIG_BLOCK, &blockset, NULL);
 	alarm(MAINTAINANCE_TIME(&s));
 
 	for (;;) {
 		check_other_work(&s);
 
 		/* initialize select */
-		FD_ZERO(&rd);
+		FD_ZERO(&rd_set);
+		FD_ZERO(&wr_set);
 
-		list_for_each(&s.llist.head, ltmp, list) {
+		list_for_each(&s.listen_list.head, ltmp, list) {
 			if (ltmp->fd == -1) continue;
 
-			val = fcntl(ltmp->fd, F_GETFL, 0);
-			if ((val == -1)
-			    || (fcntl(ltmp->fd, F_SETFL, val | O_NONBLOCK) <
-				0)) {
-				e = errno;
-				mslog(&s, NULL, LOG_ERR, "fcntl() error: %s", strerror(e));
-				exit(1);
-			}
-
-			FD_SET(ltmp->fd, &rd);
+			FD_SET(ltmp->fd, &rd_set);
 			n = MAX(n, ltmp->fd);
 		}
 
-		list_for_each(&s.clist.head, ctmp, list) {
-			FD_SET(ctmp->fd, &rd);
-			n = MAX(n, ctmp->fd);
+		list_for_each(&s.proc_list.head, ctmp, list) {
+			if (ctmp->fd > 0) {
+				FD_SET(ctmp->fd, &rd_set);
+				n = MAX(n, ctmp->fd);
+			}
 		}
 
-		tv.tv_usec = 0;
-		tv.tv_sec = 30;
-		ret = select(n + 1, &rd, NULL, NULL, &tv);
+		list_for_each(&s.ctl_list.head, ctl_tmp, list) {
+			if (ctl_tmp->enabled) {
+				if (ctl_tmp->type == CTL_READ)
+					FD_SET(ctl_tmp->fd, &rd_set);
+				else
+					FD_SET(ctl_tmp->fd, &wr_set);
+				n = MAX(n, ctl_tmp->fd);
+			}
+		}
+
+#ifdef HAVE_PSELECT
+		ts.tv_nsec = 0;
+		ts.tv_sec = 30;
+		ret = pselect(n + 1, &rd_set, &wr_set, NULL, &ts, &emptyset);
+#else
+		ts.tv_usec = 0;
+		ts.tv_sec = 30;
+		sigprocmask(SIG_UNBLOCK, &blockset, NULL);
+		ret = select(n + 1, &rd_set, &wr_set, NULL, &ts);
+		sigprocmask(SIG_BLOCK, &blockset, NULL);
+#endif
 		if (ret == -1 && errno == EINTR)
 			continue;
 
 		if (ret < 0) {
 			e = errno;
-			mslog(&s, NULL, LOG_ERR, "Error in select(): %s",
+			mslog(&s, NULL, LOG_ERR, "Error in pselect(): %s",
 			       strerror(e));
 			exit(1);
 		}
 
 		/* Check for new connections to accept */
-		list_for_each(&s.llist.head, ltmp, list) {
-			set = FD_ISSET(ltmp->fd, &rd);
+		list_for_each(&s.listen_list.head, ltmp, list) {
+			set = FD_ISSET(ltmp->fd, &rd_set);
 			if (set && ltmp->socktype == SOCK_STREAM) {
 				/* connection on TCP port */
+				memset(&ws, 0, sizeof(ws));
+
 				ws.remote_addr_len = sizeof(ws.remote_addr);
 				fd = accept(ltmp->fd, (void*)&ws.remote_addr, &ws.remote_addr_len);
 				if (fd < 0) {
@@ -825,7 +992,6 @@ int main(int argc, char** argv)
 					setproctitle(PACKAGE_NAME"-worker");
 					kill_on_parent_kill(SIGTERM);
 					
-
 					ws.config = &config;
 					ws.cmd_fd = cmd_fd[1];
 					ws.tun_fd = -1;
@@ -834,6 +1000,7 @@ int main(int argc, char** argv)
 					ws.creds = &s.creds;
 
 					/* Drop privileges after this point */
+					sigprocmask(SIG_UNBLOCK, &blockset, NULL);
 					drop_privileges(&s);
 
 					vpn_server(&ws);
@@ -852,10 +1019,11 @@ fork_failed:
 					ctmp->remote_addr_len = ws.remote_addr_len;
 
 					ctmp->pid = pid;
+					ctmp->conn_time = time(0);
 					ctmp->fd = cmd_fd[0];
 					set_cloexec_flag (cmd_fd[0], 1);
 
-					list_add(&s.clist.head, &(ctmp->list));
+					list_add(&s.proc_list.head, &(ctmp->list));
 
 					put_into_cgroup(&s, s.config->cgroup, pid);
 
@@ -879,16 +1047,31 @@ fork_failed:
 		}
 
 		/* Check for any pending commands */
-		list_for_each_safe(&s.clist.head, ctmp, cpos, list) {
-			if (FD_ISSET(ctmp->fd, &rd)) {
+		list_for_each_safe(&s.proc_list.head, ctmp, cpos, list) {
+			if (ctmp->fd >= 0 && FD_ISSET(ctmp->fd, &rd_set)) {
 				ret = handle_commands(&s, ctmp);
-				if (ret < 0) {
-					remove_from_script_list(&s, ctmp);
+				if (ret == ERR_WORKER_TERMINATED && ctmp->status == PS_AUTH_INIT &&
+					s.config->cisco_client_compat != 0) {
+					proc_to_zombie(&s, ctmp);
+				} else if (ret < 0) {
 					remove_proc(&s, ctmp, (ret!=ERR_WORKER_TERMINATED)?1:0);
 				}
 			}
 		}
 
+		/* Check for pending control commands */
+		list_for_each_safe(&s.ctl_list.head, ctl_tmp, ctl_pos, list) {
+			if (ctl_tmp->enabled == 0)
+				continue;
+
+			if (ctl_tmp->type == CTL_READ) {
+				if (FD_ISSET(ctl_tmp->fd, &rd_set))
+					ctl_handle_commands(&s, ctl_tmp);
+			} else {
+				if (FD_ISSET(ctl_tmp->fd, &wr_set))
+					ctl_handle_commands(&s, ctl_tmp);
+			}
+		}
 	}
 
 	return 0;
