@@ -27,11 +27,13 @@
 #include <net/if.h>
 #include <vpn.h>
 #include <tlslib.h>
-#include "ipc.h"
+#include "ipc.pb-c.h"
 #include <cookies.h>
+#include <common.h>
 
 int cmd_parser (int argc, char **argv, struct cfg_st* config);
 void reload_cfg_file(struct cfg_st* config);
+void clear_cfg_file(struct cfg_st* config);
 void write_pid_file(void);
 void remove_pid_file(void);
 
@@ -63,9 +65,11 @@ struct script_wait_st {
 };
 
 enum {
-	PS_AUTH_INACTIVE,
-	PS_AUTH_INIT,
-	PS_AUTH_COMPLETED,
+	PS_AUTH_INACTIVE, /* no comm with worker */
+	PS_AUTH_FAILED, /* no tried authenticated but failed */
+	PS_AUTH_INIT, /* worker has sent an auth init msg */
+	PS_AUTH_ZOMBIE, /* in INIT state but worker has disconnected! - only present when cisco-client-compat is set */
+	PS_AUTH_COMPLETED, /* successful authentication */
 };
 
 /* Each worker process maps to a unique proc_st structure.
@@ -76,6 +80,8 @@ struct proc_st {
 	pid_t pid;
 	time_t udp_fd_receive_time; /* when the corresponding process has received a UDP fd */
 	
+	time_t conn_time; /* the time the user connected */
+
 	/* the tun lease this process has */
 	struct tun_lease_st tun_lease;
 	struct ip_lease_st *ipv4;
@@ -84,11 +90,19 @@ struct proc_st {
 	struct sockaddr_storage remote_addr; /* peer address */
 	socklen_t remote_addr_len;
 
+	/* A unique session identifier used to distinguish sessions
+	 * prior to authentication. It is sent as cookie to the client
+	 * who re-uses it when it performs authentication in multiple
+	 * sessions.
+	 */
+	uint8_t sid[SID_SIZE];
+	unsigned sid_size; /* acts as a flag if sid is set */
+
 	/* The DTLS session ID associated with the TLS session 
 	 * it is either generated or restored from a cookie.
 	 */
-	uint8_t session_id[GNUTLS_MAX_SESSION_ID];
-	unsigned session_id_size; /* would act as a flag if session_id is set */
+	uint8_t dtls_session_id[GNUTLS_MAX_SESSION_ID];
+	unsigned dtls_session_id_size; /* would act as a flag if session_id is set */
 	
 	/* The following are set by the worker process (or by a stored cookie) */
 	char username[MAX_USERNAME_SIZE]; /* the owner */
@@ -96,14 +110,21 @@ struct proc_st {
 	char hostname[MAX_HOSTNAME_SIZE]; /* the requested hostname */
 	uint8_t cookie[COOKIE_SIZE]; /* the cookie associated with the session */
 
+	/* the following are copied here from the worker process for reporting
+	 * purposes (from main-ctl-handler). */
+	char user_agent[MAX_AGENT_NAME];
+	char tls_ciphersuite[MAX_CIPHERSUITE_NAME];
+	char dtls_ciphersuite[MAX_DTLS_CIPHERSUITE_NAME];
+
 	/* if the session is initiated by a cookie the following two are set
-	 * and are considered when generating an IP address.
+	 * and are considered when generating an IP address. That is used to
+	 * generate the same address as previously allocated.
 	 */
 	uint8_t seeds_are_set; /* non zero if the following two elements are set */
 	uint8_t ipv4_seed[4];
 
 	void * auth_ctx; /* the context of authentication */
-	unsigned auth_status; /* PS_AUTH_ */
+	unsigned status; /* PS_AUTH_ */
 	unsigned auth_reqs; /* the number of requests received */
 	
 	unsigned applied_iroutes; /* whether the iroutes in the config have been successfully applied */
@@ -134,6 +155,21 @@ struct ban_list_st {
 	struct list_head head;
 };
 
+#define CTL_READ 1
+#define CTL_WRITE 2
+
+struct ctl_handler_st {
+	struct list_node list;
+	int fd;
+	unsigned type; /* CTL_READ/WRITE */
+	unsigned enabled;
+	void* watch;
+};
+
+struct ctl_list_st {
+	struct list_head head;
+};
+
 typedef struct main_server_st {
 	struct cfg_st *config;
 	
@@ -146,8 +182,8 @@ typedef struct main_server_st {
 	/* tls credentials */
 	struct tls_st creds;
 
-	struct listen_list_st llist;
-	struct proc_list_st clist;
+	struct listen_list_st listen_list;
+	struct proc_list_st proc_list;
 	struct script_list_st script_list;
 	struct ban_list_st ban_list;
 	
@@ -157,6 +193,9 @@ typedef struct main_server_st {
 	unsigned active_clients;
 
 	void * auth_extra;
+
+	struct ctl_list_st ctl_list;
+	void * ctl_ctx;
 } main_server_st;
 
 void clear_lists(main_server_st *s);
@@ -167,28 +206,35 @@ int user_connected(main_server_st *s, struct proc_st* cur);
 void user_disconnected(main_server_st *s, struct proc_st* cur);
 
 void expire_tls_sessions(main_server_st *s);
-
-int send_resume_fetch_reply(main_server_st* s, struct proc_st* proc,
-		cmd_resume_reply_t r, struct cmd_resume_fetch_reply_st * reply);
+void expire_zombies(main_server_st* s);
 
 int send_udp_fd(main_server_st* s, struct proc_st * proc, int fd);
 
-int handle_resume_delete_req(main_server_st* s, struct proc_st* proc,
-  			   const struct cmd_resume_fetch_req_st * req);
+int handle_resume_delete_req(main_server_st* s, struct proc_st * proc,
+  			   const SessionResumeFetchMsg * req);
 
-int handle_resume_fetch_req(main_server_st* s, struct proc_st* proc,
-  			   const struct cmd_resume_fetch_req_st * req, 
-  			   struct cmd_resume_fetch_reply_st * rep);
+int handle_resume_fetch_req(main_server_st* s, struct proc_st * proc,
+  			   const SessionResumeFetchMsg * req, 
+  			   SessionResumeReplyMsg* rep);
 
 int handle_resume_store_req(main_server_st* s, struct proc_st *proc,
-  			   const struct cmd_resume_store_req_st * req);
+  			   const SessionResumeStoreReqMsg *);
 
 void 
 __attribute__ ((format(printf, 4, 5)))
-    mslog(const main_server_st * s, const struct proc_st* proc,
+    _mslog(const main_server_st * s, const struct proc_st* proc,
     	int priority, const char *fmt, ...);
+
+#ifdef __GNUC__
+# define mslog(s, proc, prio, fmt, ...) \
+	(prio==LOG_ERR)?_mslog(s, proc, prio, "%s:%d: "fmt, __FILE__, __LINE__, ##__VA_ARGS__): \
+	_mslog(s, proc, prio, fmt, ##__VA_ARGS__)
+#else
+# define mslog _mslog
+#endif
+
 void  mslog_hex(const main_server_st * s, const struct proc_st* proc,
-    	int priority, const char *prefix, uint8_t* bin, unsigned bin_size);
+    	int priority, const char *prefix, uint8_t* bin, unsigned bin_size, unsigned b64);
 
 int open_tun(main_server_st* s, struct proc_st* proc);
 int set_tun_mtu(main_server_st* s, struct proc_st * proc, unsigned mtu);
@@ -196,15 +242,17 @@ int set_tun_mtu(main_server_st* s, struct proc_st * proc, unsigned mtu);
 int send_auth_reply_msg(main_server_st* s, struct proc_st* proc);
 
 int send_auth_reply(main_server_st* s, struct proc_st* proc,
-			cmd_auth_reply_t r);
+			AuthReplyMsg__AUTHREP r);
 
 int handle_auth_cookie_req(main_server_st* s, struct proc_st* proc,
- 			   const struct cmd_auth_cookie_req_st * req);
+ 			   const AuthCookieRequestMsg * req);
 int generate_cookie(main_server_st *s, struct proc_st* proc);
 int handle_auth_init(main_server_st *s, struct proc_st* proc,
-		     const struct cmd_auth_init_st * req);
+		     const AuthInitMsg * req);
+int handle_auth_reinit(main_server_st *s, struct proc_st** proc,
+		     const AuthReinitMsg * req);
 int handle_auth_req(main_server_st *s, struct proc_st* proc,
-		     struct cmd_auth_req_st * req);
+		     const AuthRequestMsg * req);
 
 int check_multiple_users(main_server_st *s, struct proc_st* proc);
 
@@ -220,7 +268,30 @@ int parse_group_cfg_file(main_server_st* s, const char* file, struct group_cfg_s
 
 void del_additional_config(struct group_cfg_st* config);
 void remove_proc(main_server_st* s, struct proc_st *proc, unsigned k);
+void proc_to_zombie(main_server_st* s, struct proc_st *proc);
 
 void put_into_cgroup(main_server_st * s, const char* cgroup, pid_t pid);
 
+inline static
+int send_msg_to_worker(main_server_st* s, struct proc_st* proc, uint8_t cmd, 
+	    const void* msg, pack_size_func get_size, pack_func pack)
+{
+	mslog(s, proc, LOG_DEBUG, "sending message '%s' to worker", cmd_request_to_str(cmd));
+	return send_msg(proc->fd, cmd, msg, get_size, pack);
+}
+
+inline static
+int send_socket_msg_to_worker(main_server_st* s, struct proc_st* proc, uint8_t cmd, 
+		int socketfd, const void* msg, pack_size_func get_size, pack_func pack)
+{
+	mslog(s, proc, LOG_DEBUG, "sending (socket) message %u to worker", (unsigned)cmd);
+	return send_socket_msg(proc->fd, cmd, socketfd, msg, get_size, pack);
+}
+
+void ctl_handle_commands(main_server_st* s, struct ctl_handler_st* ctl);
+int ctl_handler_init(main_server_st* s);
+void ctl_handler_deinit(main_server_st* s);
+void request_reload(int signo);
+void request_stop(int signo);
+
 #endif
