@@ -41,6 +41,7 @@ void tls_cork(gnutls_session_t session);
 int tls_uncork(gnutls_session_t session);
 
 void tls_global_init(struct main_server_st* s);
+void tls_global_deinit(struct main_server_st* s);
 void tls_global_init_certs(struct main_server_st* s);
 
 ssize_t tls_send_file(gnutls_session_t session, const char *file);
@@ -70,6 +71,8 @@ size_t tls_get_overhead(gnutls_protocol_t, gnutls_cipher_algorithm_t, gnutls_mac
 
 void tls_close(gnutls_session_t session);
 
+unsigned tls_has_session_cert(struct worker_st * ws);
+
 void tls_fatal_close(gnutls_session_t session,
 			    gnutls_alert_description_t a);
 
