@@ -35,95 +35,43 @@
 
 #include <vpn.h>
 #include <worker.h>
-#include "ipc.h"
+#include "common.h"
+#include "ipc.pb-c.h"
 #include <cookies.h>
 #include <tlslib.h>
 
 
-static int send_resume_fetch_req(worker_st * ws, 
-				const struct cmd_resume_fetch_req_st* r,
-				int delete)
+static int recv_resume_fetch_reply(worker_st *ws, gnutls_datum_t *sdata)
 {
-	struct iovec iov[2];
-	uint8_t cmd;
-	struct msghdr hdr;
-
-	memset(&hdr, 0, sizeof(hdr));
-	
-	if (delete != 0)
-		cmd = RESUME_DELETE_REQ;
-	else
-		cmd = RESUME_FETCH_REQ;
-
-	iov[0].iov_base = &cmd;
-	iov[0].iov_len = 1;
-
-	iov[1].iov_base = (void*)r;
-	iov[1].iov_len = sizeof(*r);
-	
-	hdr.msg_iov = iov;
-	hdr.msg_iovlen = 2;
-
-	return(sendmsg(ws->cmd_fd, &hdr, 0));
-}
-
-static int send_resume_store_req(worker_st * ws, const struct cmd_resume_store_req_st* r)
-{
-	struct iovec iov[2];
-	uint8_t cmd;
-	struct msghdr hdr;
-
-	memset(&hdr, 0, sizeof(hdr));
-	
-	cmd = RESUME_STORE_REQ;
-
-	iov[0].iov_base = &cmd;
-	iov[0].iov_len = 1;
-
-	iov[1].iov_base = (void*)r;
-	iov[1].iov_len = sizeof(*r);
-	
-	hdr.msg_iov = iov;
-	hdr.msg_iovlen = 2;
-
-	return(sendmsg(ws->cmd_fd, &hdr, 0));
-}
-
-static int recv_resume_fetch_reply(worker_st *ws, struct cmd_resume_fetch_reply_st* resp)
-{
-	struct iovec iov[2];
-	uint8_t cmd = 0;
-	struct msghdr hdr;
 	int ret;
+	SessionResumeReplyMsg *resp;
+
+	ret = recv_msg( ws->cmd_fd, RESUME_FETCH_REP, (void*)&resp, 
+		(unpack_func)session_resume_reply_msg__unpack);
+	if (ret < 0) {
+		oclog(ws, LOG_ERR, "error receiving resumption reply (fetch)");
+		return ret;
+	}
+
+	if (resp->reply != SESSION_RESUME_REPLY_MSG__RESUME__REP__OK) {
+		ret = -1;
+		goto cleanup;
+	}
+		
+	sdata->data = gnutls_malloc(resp->session_data.len);
+	if (sdata->data == NULL) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	sdata->size = resp->session_data.len;
+	memcpy(sdata->data, resp->session_data.data, sdata->size);
+
+	ret = 0;
+cleanup:
+	session_resume_reply_msg__free_unpacked(resp, NULL);
 	
-	iov[0].iov_base = &cmd;
-	iov[0].iov_len = 1;
-
-	iov[1].iov_base = resp;
-	iov[1].iov_len = sizeof(*resp);
-
-	memset(&hdr, 0, sizeof(hdr));
-	hdr.msg_iov = iov;
-	hdr.msg_iovlen = 2;
-
-	ret = recvmsg( ws->cmd_fd, &hdr, 0);
-	if (ret <= sizeof(*resp)) {
-		int e = errno;
-		oclog(ws, LOG_ERR, "resume_fetch_reply: incorrect data (%d, expected %d) from main: %s", ret, (int)sizeof(*resp)+1, strerror(e));
-		return -1;
-	}
-
-	if (cmd != RESUME_FETCH_REP) {
-		oclog(ws, LOG_ERR, "resume_fetch_reply: unexpected response (%d, expected %d) from main", (int)cmd, (int)RESUME_FETCH_REP);
-		return -1;
-	}
-
-	switch(resp->reply) {
-		case REP_RESUME_OK:
-			return 0;
-		default:
-			return -1;
-	}
+	return ret;
 }
 
 /* sends an authentication request to main thread and waits for
@@ -133,43 +81,26 @@ static int recv_resume_fetch_reply(worker_st *ws, struct cmd_resume_fetch_reply_
 static gnutls_datum_t resume_db_fetch(void *dbf, gnutls_datum_t key)
 {
 worker_st *ws = dbf;
-struct cmd_resume_fetch_req_st areq;
-struct cmd_resume_fetch_reply_st *arep;
 gnutls_datum_t r = { NULL, 0 };
 int ret;
+SessionResumeFetchMsg msg = SESSION_RESUME_FETCH_MSG__INIT;
 
 	if (key.size > GNUTLS_MAX_SESSION_ID) {
-		oclog(ws, LOG_DEBUG, "Session ID size exceeds the maximum %u", key.size);
+		oclog(ws, LOG_DEBUG, "session ID size exceeds the maximum %u", key.size);
 		return r;
 	}
 
-	areq.session_id_size = key.size;
-	memcpy(areq.session_id, key.data, key.size);
+	msg.session_id.len = key.size;
+	msg.session_id.data = key.data;
 
-	oclog(ws, LOG_DEBUG, "sending resumption request (fetch)");
-
-	ret = send_resume_fetch_req(ws, &areq, 0);
+	ret = send_msg_to_main(ws, RESUME_FETCH_REQ, &msg,
+		(pack_size_func)session_resume_fetch_msg__get_packed_size,
+		(pack_func)session_resume_fetch_msg__pack);
 	if (ret < 0)
 		return r;
 
-	arep = malloc(sizeof(*arep));
-	if (arep == NULL)
-		return r;
-
-	ret = recv_resume_fetch_reply(ws, arep);
-	if (ret < 0) {
-		goto cleanup;
-	}
-		
-	r.data = gnutls_malloc(arep->session_data_size);
-	if (r.data == NULL)
-		goto cleanup;
-
-	r.size = arep->session_data_size;
-	memcpy(r.data, arep->session_data, r.size);
-
-cleanup:
-	free(arep);
+	recv_resume_fetch_reply(ws, &r);
+	
 	return r;
 }
 
@@ -178,7 +109,7 @@ static int
 resume_db_store (void *dbf, gnutls_datum_t key, gnutls_datum_t data)
 {
 worker_st *ws = dbf;
-struct cmd_resume_store_req_st areq;
+SessionResumeStoreReqMsg msg = SESSION_RESUME_STORE_REQ_MSG__INIT;
 int ret;
 
 	if (data.size > MAX_SESSION_DATA_SIZE) {
@@ -191,13 +122,15 @@ int ret;
 		return GNUTLS_E_DB_ERROR;
 	}
 
-	areq.session_id_size = key.size;
-	areq.session_data_size = data.size;
+	msg.session_id.len = key.size;
+	msg.session_data.len = data.size;
 
-	memcpy(areq.session_id, key.data, key.size);
-	memcpy(areq.session_data, data.data, data.size);
+	msg.session_id.data = key.data;
+	msg.session_data.data = data.data;
 
-	ret = send_resume_store_req(ws, &areq);
+	ret = send_msg_to_main(ws, RESUME_STORE_REQ, &msg,
+		(pack_size_func)session_resume_store_req_msg__get_packed_size,
+		(pack_func)session_resume_store_req_msg__pack);
 	if (ret < 0) {
 		return GNUTLS_E_DB_ERROR;
 	}
@@ -212,20 +145,20 @@ int ret;
 static int resume_db_delete(void *dbf, gnutls_datum_t key)
 {
 worker_st *ws = dbf;
-struct cmd_resume_fetch_req_st areq;
 int ret;
+SessionResumeFetchMsg msg = SESSION_RESUME_FETCH_MSG__INIT;
 
 	if (key.size > GNUTLS_MAX_SESSION_ID) {
 		oclog(ws, LOG_DEBUG, "Session ID size exceeds the maximum %u", key.size);
 		return GNUTLS_E_DB_ERROR;
 	}
 
-	areq.session_id_size = key.size;
-	memcpy(areq.session_id, key.data, key.size);
+	msg.session_id.len = key.size;
+	msg.session_id.data = key.data;
 
-	oclog(ws, LOG_DEBUG, "sending resumption request (delete)");
-
-	ret = send_resume_fetch_req(ws, &areq, 1);
+	ret = send_msg_to_main(ws, RESUME_DELETE_REQ, &msg,
+		(pack_size_func)session_resume_fetch_msg__get_packed_size,
+		(pack_func)session_resume_fetch_msg__pack);
 	if (ret < 0)
 		return GNUTLS_E_DB_ERROR;
 
