@@ -30,6 +30,7 @@
 #include <vpn.h>
 #include <cookies.h>
 #include <tlslib.h>
+#include <common.h>
 #include <str.h>
 
 typedef enum {
@@ -49,6 +50,8 @@ enum {
 	HEADER_CSTP_ATYPE,
 	HEADER_DTLS_MTU,
 	HEADER_DTLS_CIPHERSUITE,
+	HEADER_CONNECTION,
+	HEADER_USER_AGENT,
 };
 
 enum {
@@ -64,6 +67,15 @@ enum {
 	S_AUTH_COMPLETE
 };
 
+typedef struct dtls_ciphersuite_st {
+	const char* oc_name;
+	const char* gnutls_name; /* the gnutls priority string to set */
+	unsigned server_prio; /* the highest the more we want to negotiate that */
+	unsigned gnutls_cipher;
+	unsigned gnutls_mac;
+	unsigned gnutls_version;
+} dtls_ciphersuite_st;
+
 struct http_req_st {
 	char url[256];
 
@@ -72,20 +84,19 @@ struct http_req_st {
 	unsigned int header_state;
 
 	char hostname[MAX_HOSTNAME_SIZE];
+	char user_agent[MAX_AGENT_NAME];
 	unsigned int next_header;
 	unsigned char cookie[COOKIE_SIZE];
 	unsigned int cookie_set;
+	unsigned int sid_cookie_set;
+
 	unsigned char master_secret[TLS_MASTER_SIZE];
 	unsigned int master_secret_set;
 
 	char *body;
 	unsigned int body_length;
 
-	char *gnutls_ciphersuite; /* static string */
-	char *selected_ciphersuite; /* static string */
-	int gnutls_cipher;
-	int gnutls_mac;
-	int gnutls_version;
+	const dtls_ciphersuite_st *selected_ciphersuite;
 
 	unsigned int headers_complete;
 	unsigned int message_complete;
@@ -100,6 +111,10 @@ typedef struct worker_st {
 	struct tls_st *creds;
 	gnutls_session_t session;
 	gnutls_session_t dtls_session;
+
+	/* inique session identifier */
+	uint8_t sid[SID_SIZE];
+
 	int cmd_fd;
 	int conn_fd;
 	
@@ -119,7 +134,10 @@ typedef struct worker_st {
 	/* set after authentication */
 	int udp_fd;
 	udp_port_state_t udp_state;
-	
+
+	/* protection from multiple rehandshakes */
+	time_t last_tls_rehandshake;
+
 	/* for mtu trials */
 	unsigned last_good_mtu;
 	unsigned last_bad_mtu;
@@ -163,7 +181,15 @@ void set_resume_db_funcs(gnutls_session_t);
 
 
 void __attribute__ ((format(printf, 3, 4)))
-    oclog(const worker_st * server, int priority, const char *fmt, ...);
+    _oclog(const worker_st * server, int priority, const char *fmt, ...);
+
+#ifdef __GNUC__
+# define oclog(server, prio, fmt, ...) \
+	(prio==LOG_ERR)?_oclog(server, prio, "%s:%d: "fmt, __FILE__, __LINE__, ##__VA_ARGS__): \
+	_oclog(server, prio, fmt, ##__VA_ARGS__)
+#else
+# define oclog _oclog
+#endif
 
 int get_rt_vpn_info(worker_st * ws,
                     struct vpn_st* vinfo, char* buffer, size_t buffer_size);
@@ -172,4 +198,12 @@ int send_tun_mtu(worker_st *ws, unsigned int mtu);
 int handle_worker_commands(struct worker_st *ws);
 int disable_system_calls(struct worker_st *ws);
 
+inline static
+int send_msg_to_main(worker_st *ws, uint8_t cmd, 
+	    const void* msg, pack_size_func get_size, pack_func pack)
+{
+	oclog(ws, LOG_DEBUG, "sending message '%s' to main", cmd_request_to_str(cmd));
+	return send_msg(ws->cmd_fd, cmd, msg, get_size, pack);
+}
+
 #endif
